<?php

if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

/**
 * Classe CEdit
 *
 * Chaque composant ACS peut définir une classe <MonComposant>
 * qui étend la classe CEdit en définissant des méthodes de l'interface ICEdit
 * Ce sont des points d'entrée logiciels pour les objets de classe CEdit,
 * un peu comparables aux pipelines spip, mais en technologie objet explicite
 */
abstract class CEdit implements ICEdit {
}

interface ICEdit {
	public function update();
}

/**
 * Classe AdminComposant
 *
 * Interface d'admin de composants pour ACS
 */
class AdminComposant {
	/**
	 * Constructeur
	 * Instancie (au besoin) un objet CEdit à l'exécution
	 * pour en adopter les méthodes implémentées.
	 * @param string $class classe du composant
	 * @param integer $nic numero d'instance du composant
	 */
	public function __construct($class, $nic = 0) {
		global $_POST;

		include_spip('inc/xml'); // spip_xml_load()
		include_spip('inc/traduire');
		include_spip('inc/acs_version');

		$this->exists = false;
		$this->class = $class; // Classe de composant (nom)
		$this->nic = $nic != 0 ? $nic : ''; // Numéro d'instance du composant
		$this->fullname = 'acs' . ucfirst($class) . $this->nic;
		$this->errors = [];
		$this->vars = [];
		$this->cvars = []; // Variables issues d'un autre composant
		$this->necessite = []; // Dependances du composant (version spip, plugins et composants)
		$this->necessite['plugin'] = []; // Dependances du composant (plugins)
		$this->necessite['composant'] = []; // Dependances du composant (composants)
		$this->nb_widgets = 0; // Nb de variables de type widget
		$this->root = find_in_path('composants/' . $class);// Dossier racine du composant
		$this->icon = $this->root . '/images/' . $class . '_icon.gif';
		if (!is_readable($this->icon)) {
			$this->icon = _DIR_PLUGIN_ACS . '/images/composant-24.gif';
		}
		$this->optionnel = 'oui';
		$this->enable = true;

		// Lit les paramètres de configuration du composant, en tenant compte d'un overide eventuel
		if (is_readable($this->root . '/ecrire/composant.xml')) {
			$config = spip_xml_load($this->root . '/ecrire/composant.xml');
		} else {
			$config = spip_xml_load(find_in_path('composants/' . $class . '/ecrire/composant.xml'));
		}
		// si $config est vide, le composant n'existe pas, et on s'arrête là.
		if (!$config) {
			return false;
		} else {
			$this->exists = true;
		}

		// Affecte ses paramètres de configuration à l'objet Composant
		$c = $config['composant'][0];
		$this->nom = $c['nom'][0];
		$this->version = $c['version'][0];
		$this->group = $c['group'][0];

		// Lit les dépendances (plugins: necessite, composants: necessite_composant)
		if (spip_xml_match_nodes(',^necessite,', $c, $needs)) {
			foreach (array_keys($c) as $tag) {
				list($tag,$att) = spip_xml_decompose_tag($tag);
				if ($att) {
					$this->necessite[($att['set'] ? 'composant' : 'plugin')][] = $att;
				}
			}
			krsort($this->necessite); // on met les plugins avant les composants
		}

		if (is_array($c['param'])) {
			foreach ($c['param'] as $param) {
				if (is_array($param) && is_array($param['nom']) && is_array($param['valeur'])) {
					$pn = $param['nom'][0];
					$this->$pn = $param['valeur'][0];
				}
				else {
					$pn = $param['nom'][0];
					$this->$pn = true;
				}
			}
		}

		// Active le composant non optionnel, si nécessaire
		if (($this->optionnel == 'non') || ($this->optionnel == 'no') || ($this->optionnel == 'false')) {
			if (acs_get($this->fullname . 'Use') != 'oui') {
				acs_set($this->fullname . 'Use', 'oui');
				$this->enable = true;
				$updated = true;
			}
		}
		else { // Regarde si le composant optionnel doit être activé
			// Désactive le composant s'il dépend de plugins non activés
			if (strpos($this->optionnel, 'plugin') === 0) {
				$plugins_requis = explode(' ', substr($this->optionnel, 7));
				foreach ($plugins_requis as $plug) {
					$plug = strtoupper(trim($plug));
					if (!acs_get_from_active_plugin($plug)) {
						$this->enable = false;
					}
				}
			}
			// Désactive le composant si "optionnel" est égal à une variable de configuration non égale à "oui"
			// (si optionnel ne vaut pas oui, yes, ou true, il s'agit d'un nom de variable meta. Exemple: activer_breves)
			elseif (
				isset($this->optionnel) &&
				($this->optionnel != 'oui') &&
				($this->optionnel != 'yes') &&
				($this->optionnel != 'true') &&
				(acs_get($this->optionnel) != 'oui')
			) {
				acs_set($this->fullname . 'Use', 'non');
				$this->enable = false;
				$updated = true;
			}
		}

		$this->vars[0] = [
			'nom' => 'Use',
			'valeur' => acs_get($this->fullname . 'Use')
		];

		if (_ACS_LOG >= _LOG_DEBUG) {
			$this->errors[] = 'Debug mode _ACS_LOG = ' . _ACS_LOG . ' in acs_options.php<br />';
		}

		if (is_array($c['variable'])) {
			foreach ($c['variable'] as $k => $var) {
				if (!is_array($var)) {
					continue; // Peut se produire en cas d'erreur dans composant.xml
				}
				foreach ($var as $varname => $value) {
					if (count($value) > 1) {
							$v = $value;
					} else {
						$v = $value[0];
					}
					if (_ACS_LOG >= _LOG_DEBUG) {
						$this->errors['vars'] .= $class . ($nic ? '#' . $nic : '') . '->vars[' . $k . '][' . $varname . '] = ' . htmlentities((is_array($v) ? 'Array(' . implode($v, ', ') . ')' : $v)) . "<br />\n"; // dbg composant
					}
					$this->vars[$k + 1][$varname] = $v;
					if (($varname == 'type') && ($v == 'widget')) {
						$this->nb_widgets++;
					}
					if ($varname == 'valeur') { // Default values
						if (substr($v, 0, 4) == '=acs') {
							if (!in_array($v, $this->cvars)) {
								$this->cvars[ ] = $v;
							}
							$v = acs_get($v);
						}
					}
				}
			}
		}

		// Mise à jour
		if ((_ACS_LOG >= _LOG_DEBUG) && $_POST) {
			include_spip('balise/acs_balises');
			$this->errors[] = "\n<br />\$_POST=\n";
			$this->errors[] = dbg($_POST, 'html');
			$this->errors[] = "\n<br />\n";
		}
		// Suppression de l'instance du composant
		if (_request('del_composant') == 'delete') {
			foreach ($this->vars as $var) {
				$v = $this->fullname . $var['nom'];
				effacer_meta($v);
			}
			$updated = true;
		}
		// Mise à jour
		if (_request('maj_composant_' . md5($this->fullname)) == 'oui') {
			foreach ($this->vars as $var) {
				unset($nv);
				$v = $this->fullname . $var['nom'];
				switch ($var['type']) {
					case 'bord':
						// on ne traite surtout pas les variables non postées
						if (!isset($_POST[$v . 'Color'])) {
							continue 2;
						}
						if ($_POST[$v . 'Color'] === '') {
							$nv = '';
						}
						else {
							$posted_color = $_POST[$v . 'Color'];
							// si la valeur postee commence par "=", c'est une référence à la valeur d'un autre composant
							if (substr($posted_color, 0, 1) == '=') {
								$nv = $posted_color;
							}
							else {
								$nv = [
									'Width' => $_POST[$v . 'Width'],
									'Style' => $_POST[$v . 'Style'],
									'Color' => $_POST[$v . 'Color']
								];
							}
						}
						if (is_array($nv)) {
							$nv = serialize($nv);
						}
						break;

					case 'key':
						$group = $_POST[$v . 'Group'];
						$key = $_POST[$v . 'Key'];
						$nv = serialize(['Group' => $group, 'Key' => $key]);
						break;

					default:
						// on ne traite surtout pas les variables non postées
						if (!isset($_POST[$v])) {
							continue 2;
						}
						$nv = $_POST[$v];
				}

				// On continue si rien à faire
				if ($nv == acs_get($v)) {
					continue;
				}

				// Si la nouvelle valeur est vide
				if ($nv === '') {
					// et que la variable n'a jamais été initialisée,
					if ((null !== $var['valeur']) && (null !== acs_get($v))) {
						// on lui affecte la valeur par défaut
						if (substr($var['valeur'], 0, 4) == '=acs') {
							$nv = acs_get($var['valeur']);
						} else {
							$nv = $var['valeur'];
						}
					}
				}

				// Mise à jour
				acs_set($v, $nv);
				$updated = true;
				acs_log('AdminComposant updated: $v = ' . $v . ', $nv = ' . $nv, _LOG_DEBUG);
			}
		}
		if (isset($updated)) {
			if (isset($this->update)) {
				include_spip('composants/' . $class . '/ecrire/' . $class);
				$c_obj = 'acs' . ucfirst($class) . 'Edit';
				if (class_exists($c_obj)) {
					$$c_obj = new $c_obj();
					if (($$c_obj instanceof CEdit) && is_callable([$$c_obj, 'update'])) {
						if (!$$c_obj->update()) {
							$this->errors[] = $c_obj . '->update ' . _T('acs:failed') . ' ' . implode(' ', $$c_obj->errors);
						}
					} else {
						$this->errors[] = $c_obj . '->update ' . _T('acs:not_callable');
					}
				}	else {
					$this->errors[] = $c_obj . '->update ' . _T('acs:not_found');
				}
			}
			acs_recalcul();
			unset($updated);
		}
		acs_log('New AdminComposant($class=' . $class . ', $nic=' . $nic . ', _ACS_LOG=' . _ACS_LOG . ')', _LOG_DEBUG);
	}

/**
 * Méthode privée _getCvarsHtml: retourne du code html pour les variables du composant
 * faisant référence à une variable définie par un autre composant
 * pour leurs valeurs par défaut
 */
	private function _getCvarsHtml() {
		foreach ($this->cvars as $k => $var) {
			if (null !== acs_get($var)) {
				$class = ' alert';
			} else {
				$class = '';
			}
			$this->cvars[$k] =
			'<a class="nompage' . $class . '" title="' . acs_get($var) . '">' .
				substr($this->cvars[$k], 3) . '</a>';
		}
		return implode(', ', $this->cvars);
	}

	public function t($string) {
		$s = $this->class . '_' . $string;
		$t = _T('acs:' . $s);

		// On retourne la chaine si elle n'a pas été traduite (test pré-SPIP 3) :
		if ($t == str_replace('_', ' ', $s)) {
			return $string;
		}
		// On s'adapte à SPIP 3 :
		if (substr($t, 0, 12) == '<blink style') {
			return $string;
		}
		// On retourne la traduction
		return str_replace('"', '&quot;', $t);
	}

/**
 * Méthode gauche: affiche la colonne gauche dans spip admin
 * @return string html code
 */
	public function gauche() {
		global $spip_version_code;
		if (!$this->exists) {
			return '<div class="alert" style="text-align: center"><b>' .
				ucfirst($this->class) .
			'</b> ' . _T('acs:not_found') . '</div>';
		}
		if ($this->t('description') != 'description') {
			$r .= '<div>' . $this->t('description') . '</div><br />';
		}

		if ($this->t('info') != 'info') {
			$r .= '<div class="onlinehelp" style="text-align: justify">' . $this->t('info') . '</div><br />';
		}

		$n = 999;
		$r .= '<div class="onlinehelp">' .
		acs_plieur(
			'plieur_pu' . $n,
			'pu' . $n,
			'#',
			false,
			'if (typeof done' . $n . ' == \'undefined\') {
console.log(\'ajax load\');
				AjaxSqueezeNode(\'?exec=composant_get_infos&c=' . $this->class . ($this->nic ? '&nic=' . $this->nic : '') . '\', jQuery(\'#puAjax' . $n . '\'));
				jQuery(this).toggleClass(\'plon\');
				done' . $n . ' = true;
console.log(\'ajax loaded\');
			}',
			_T('acs:dev_infos')
		) . '</div><div class="pu' . $n . '">';

		if (count($this->cvars)) {
			$r .= '<br /><div class="onlinehelp">' . _T('acs:references_autres_composants') .
				'</div><div class="onlinehelplayer">' . $this->_getCvarsHtml() . '</div>';
		}
		$r .= '<div id="puAjax' . $n . '" class="puAjax' . $n . '"></div>';
		$r .= '</div>';
		return $r;
	}

/**
 * Méthode edit: affiche un editeur pour les variables du composant
 * @param mode : mode d'affichage (espace prive ou controleur)
 * @return string html code
 */
	public function edit($mode = false) {
		if (!$this->exists) {
			return '';
		}
		include_spip('public/assembler');
		include_spip('inc/composant/classControles');
		$r = '<script src="' . _DIR_PLUGIN_ACS . 'inc/picker/picker.js"></script>';
		$r .= '<input type="hidden" name="maj_composant_' . md5($this->fullname) . '" value="oui" />' .
					'<input type="hidden" name="composant" value="' . $this->class . '" />' .
					'<input type="hidden" name="nic" value="' . $this->nic . '" />';

		$varconf = $this->fullname . 'Config';
		if (($mode != 'controleur') && ($this->optionnel != 'non') && ($this->optionnel != 'no') && ($this->optionnel != 'false')) {
			$varname = $this->fullname . 'Use';
			if ((null !== acs_get($varname)) && acs_get($varname)) {
				$var = acs_get($varname);
			} else {
				$var = 'non';
			}
			$this->display = 'display: ' . ($var == 'oui' ? 'block' : 'none');
			$nc = $this->t('nom');
			if ($nc == str_replace('_', ' ', $this->t('nom'))) {
				$nc = ucfirst($this->nom);
			}
			$r .= '<div align="' . $GLOBALS['spip_lang_right'] . '"><label>' . _T('acs:use') . ' ' . $nc . ' ' . $this->nic . ' : </label>';
			// NB: acs_bouton_radio($nom, $valeur, $titre, $actif=false, $onClick="", $enable=true)
			$r .= acs_bouton_radio($varname, 'oui', _T('acs:oui'), $var == 'oui', "changeVisible(this.checked, '$varconf', 'block', 'none');", $this->enable);
			$r .= acs_bouton_radio($varname, 'non', _T('acs:non'), $var == 'non', "changeVisible(this.checked, '$varconf', 'none', 'block');", $this->enable);
			$r .= '</div>';
		}
		$r .= '<div id="' . $varconf . '" ' . (isset($this->display) ? 'style="' . $this->display . '"' : '') . ' class="c_config">';
		if (($mode != 'controleur') && isset($this->preview) && ($this->preview != 'non')	&& ($this->preview != 'no') && ($this->preview != 'false')) {
			$url = '../?page=wrap&c=composants/' . $this->class . '/' . $this->class . ($this->nic ? '&amp;nic=' . $this->nic : '') . '&v=' . acs_get('acsDerniereModif') . '&var_mode=recalcul';
		switch ($this->preview_type) {
			case 'inline':
				 require_once _DIR_ACS . 'balise/acs_balises.php';
				 $preview = '<script src="../spip.php?page=acs.js"></script><link rel="stylesheet" href="../spip.php?page=' . acs_get('ACS_SET') . '.css" type="text/css" media="screen" /><div id="' . $this->fullname . '" style="border:0;overflow: auto; width: 100%; height: ' . (is_numeric($this->preview) ? $this->preview : 80) . 'px">' . recuperer_fond('vues/composant', [
				 'c' => 'composants/' . $this->class . '/' . $this->class,
				 'nic' => $this->nic,
				 'lang' => $GLOBALS['spip_lang']
				 ]) . '</div>';
				break;
			default:
				$preview = '<iframe id="' . $this->fullname . '" width="100%" height="' . (is_numeric($this->preview) ? $this->preview : 80) . 'px" frameborder="0" style="border:0; background:' . acs_get('acsFondColor') . '" src="' . $url . '"></iframe>';
		}
		$r .= '<fieldset class="apercu"><legend><a href="javascript:void(0)" onclick="document.getElementById(\'' . $this->fullname . '\').src=\'' . $url . '\';return false;" title="' . _T('admin_recalculer') . '">' . _T('previsualisation') . '</a></legend>' . $preview . '</fieldset>';
		}
		// Affiche les variables paramétrables du composant:
		$controls = [];
		foreach ($this->vars as $var) {
			if ($var['nom'] === 'Use') {
				continue;
			}
			$v = $var['nom'];
			if (null != acs_get($this->fullname . $v)) {
				$$v = acs_get($this->fullname . $v);
			}
			elseif (isset($var['valeur'])) {
				$default = $var['valeur'];
				if ((substr($default, 0, 3) == 'acs') && (null !== acs_get($default))) {
					$$v = acs_get($default);
				}
				elseif (substr($default, 0, 3) != 'acs') {
					$$v = $default;
				}
			}
			$draw = 'ctl' . ucfirst($var['type']);
			$ctl = 'ctl' . ucfirst($var['type']);
			if (class_exists($ctl)) {
				$ctl = new $ctl($this->class, $this->nic, $v, $$v, $var);
				if (method_exists($ctl, 'draw')) {
					$controls[$var['nom']] = $ctl->draw();
				}
			} else {
				$controls[$var['nom']] = $ctl . '() undefined.<br />';
			}
		}

		// Recherche une mise en page et y remplace les variables par des contrôles, sinon ajoute les contrôles non mis en page
		if (find_in_path('composants/' . $this->class . '/ecrire/' . $this->class . '_mep.html')) {
			$mep .= recuperer_fond('composants/' . $this->class . '/ecrire/' . $this->class . '_mep', ['lang' => $GLOBALS['spip_lang'], 'nic' => $this->nic]);
			foreach ($controls as $nom => $html) {
				$tag = '&' . $nom . '&';
				$mep = str_replace($tag, $html, $mep);
			}
			// en mode controleur
			if ($mode == 'controleur') {
				$mep = preg_replace('%<admin>(.*?)</admin>%s', '', $mep);
				// on ajoute si besoin est la liste des widgets, invisible
				if ($this->nb_widgets > 0) {
					$mep .= liste_widgets(false);
				}
			}
		} else {
			foreach ($controls as $nom => $html) {
				$r .= $html;
			}
		}

		if (isset($mep)) {
			$r .= '<div align="' . $GLOBALS['spip_lang_right'] . '">' . $mep . '</div>';
		}
		$r .= '</div>';

		return $r;
	}
/**
 * Méthode help: retourne l'aide d'une variable ou d'un composant
 */
	public function help($var = null) {
		$help_src = 'acs:' . $this->class . ($var ? '_' . $var : '') . '_help';
		$help = _T($help_src);
		if ($help != $this->class . ($var ? ' ' . $var : '') . ' help') {
			return $help;
		} else {
			return false;
		}
	}
/**
 * Méthode nextInstance: retourne un numéro d'instance de composant inutilisé
 */
	public function nextInstance() {
		$i = composant_instances($this->class);
		return count($i) + 1;
	}

	/**
	 * Méthode getTraductions: retourne un tableau des traductions d'un composant
	 */
	public function getTraductions() {
		$c = $this->class;
		$r[0] = $this->_getFiles('composants/' . $c . '/lang', $ext = 'php', strlen($c) + 1);
		$r[1] = $this->_getFiles('composants/' . $c . '/ecrire/lang', $ext = 'php', strlen($c) + strlen('ecrire') + 2);
		return $r;
	}

	/**
	 * Méthode getFiles: retourne un tableau de fichiers d'un composant
	 * dont l'extendion vérifie l'expression régulière $ext (par défaut: les fichiers html)
	 * @param string $chemin
	 * @param string $ext expression régulière
	 * @param integer $skip
	 */
	private function _getFiles($chemin = '', $ext = 'html', $skip = 0) {
		$c = $this->class;
		$files = [];
		if (!$c) {
			return $files;
		}
		$dir = find_in_path($chemin);
		if (@is_dir($dir) and @is_readable($dir) and $d = @opendir($dir)) {
			while (($f = readdir($d)) !== false && ($nbfiles < 1000)) {
				// ignorer . .. .s
				if ($f[0] != '.' and $f != 'remove.txt' and @is_readable($p = "$dir/$f")) {
					if (is_file($p)) {
						if (preg_match(';.*[.]' . $ext . '$;iS', $f)) {
							$files[] = substr($f, $skip, - (strlen($ext) + 1));
						}
					}
				}
				$nbfiles++;
			}
		}
		sort($files);
		return $files;
	}

	/**
	* Méthode _getPages: retourne un tableau des squelettes qui utilisent le composant $nic
	* @param string $chemin chemin relatif d'un set de composants
	* @return array
	*/
	private function _getPages($chemin = '') {
		$c = $this->class;
		$nic = $this->nic;
		$pages = [];
		$pages['composant'] = [];
		$pages['variables'] = [];

		if (!$c) {
			return $pages;
		}
		if ($chemin == '') {
			$dir = _DIR_PLUGIN_ACS . 'sets/' . acs_get('ACS_SET');
		} else {
			$dir = find_in_path($chemin);
		}
		if (@is_dir($dir) and @is_readable($dir) and $d = @opendir($dir)) {
			$vars = $this->_getVars();
			if ($nic) {
				$cpreg = '/\{fond=composants\/' . $c . '\/[^\}]*\}.*\{nic=' . $nic . '\}/';
			} else {
				$cpreg = '/\{fond=composants\/' . $c . '\/[^\}]*\}.^[\{nic=' . $nic . '\}]*/';
			}
			while (($f = readdir($d)) !== false && ($nbfiles < 1000)) {
				if ($f[0] != '.' and $f != 'remove.txt' and @is_readable($p = "$dir/$f")) {
					if (is_file($p)) {
						if (preg_match(';.*[.]html$;iS', $f)) {
							$fic = @file_get_contents($p);
							if (preg_match($cpreg, $fic, $matches)) {
								$pages['composant'][] = substr($f, 0, -5);
							}
							foreach ($vars as $var) {
								if (strpos($fic, $var)) {
									$pages['variables'][substr($f, 0, -5)][] = $var;
								}
							}
						}
					}
				}
				$nbfiles++;
			}
			$pages['chemin'] = $chemin;
		}
		return $pages;
	}

	/**
	 * Méthode _getVars: retourne un tableau des variables du composant
	 */
	private function _getVars() {
		$composant = $this->class;
		$nic = $this->nic;
		$r = [];
		// Lit les paramètres de configuration du composant
		include_once('inc/xml.php');
		$configfile = find_in_path('composants/' . $composant . '/ecrire/composant.xml');
		$config = spip_xml_load($configfile);
		$c = $config['composant'][0];
		if (is_array($c['param'])) {
			foreach ($c['param'] as $param) {
				if (
					is_array($param['nom']) &&
					$param['nom'][0] == 'optionnel' &&
					(($param['valeur'][0] == 'oui') || ($param['valeur'][0] == 'true'))
				) {
					array_push($r, 'acs' . ucfirst($composant) . 'Use');
				}
			}
		}

		if (is_array($c['variable'])) {
			foreach ($c['variable'] as $k => $var) {
				foreach ($var as $varname => $value) {
					if ($varname == 'nom') {
						array_push($r, 'acs' . ucfirst($composant) . $nic . $value[0]);
					}
				}
			}
		}
		return $r;
	}

	/**
	 * Retourne toutes les pages des squelettes, composants, et noisettes qui
	 * contiennent l'instance $nic du composant $c
	 */
	public function listePages() {
		$nic = $this->nic;
		$c = $this->class;
		$dirs = [
			['', _T('acs:page'), _T('acs:pages')],
			['modeles', _T('acs:modele'), _T('acs:modeles')],
			['formulaires', _T('acs:formulaire'), _T('acs:formulaires')]
		];
		foreach (composants_liste() as $class => $composant) {
			$dirs[] = ['composants/' . $class, _T('acs:composant'), _T('acs:composants')];
		}
		foreach ($dirs as $dir) {
			$p = $this->_getPages($dir[0]);
			if (count($p['composant']) > 0) {
				$r = '<span class="onlinehelp">' . (count($p['composant']) > 1 ? $dir[2] : $dir[1]) . '</span> ';
				foreach ($p['composant'] as $page) {
					$r .= $this->_showOverride($p['chemin'], $page) . ' ';
				}
				$r .= '<br />';
			}
			if (count($p['variables']) > 0) {
				$len = strlen('acs' . $c . $nic);
				$r .= '<i>';
				foreach ($p['variables'] as $page => $var) {
					$r .= $this->_showOverride($p['chemin'], $page) . ' (';
					foreach ($var as $v) {
						$r .= '<a title="' . $v . (acs_get($v) ? ' = ' . htmlentities(acs_get($v)) : '') . '">' . substr($v, $len) . '</a> ';
					}
					$r = rtrim($r);
					$r .= ')<br />';
				}
				$r .= '</i>';
			}
		}
		return $r;
	}

	private function _showOverride($chemin, $page) {
		if ($chemin) {
			$chemin .= '/';
		}
		if ((null !== acs_get('ACS_OVER')) && is_readable('../' . acs_get('ACS_OVER') . '/' . $page . '.html')) {
			$r = '<u>' . $page . '</u>';
		}	else {
			$r = $page;
		}
		$r = '&nbsp;&nbsp;&nbsp;<a class="nompage" href="?exec=acs&onglet=pages&pg=' . $chemin . $page . '" title="' . $chemin . $page . '">' . $r . '</a>';
		return $r;
	}
}
